import logging
import subprocess

from pubtools.exodus.task import ExodusTask

LOG = logging.getLogger("pubtools-exodus")
LOG_FORMAT = "%(asctime)s [%(levelname)-8s] %(message)s"


class ExodusPushTask(ExodusTask):
    """Push a directory to the Exodus CDN"""

    def add_args(self):
        super(ExodusPushTask, self).add_args()

        self.parser.add_argument(
            "sync",
            nargs="+",
            help=(
                """A comma-separated pair of paths to sync: SRC,DEST.\n"""
                """Where SRC is a local path to a file or directory and DEST\n"""
                """is the remote destination for sync."""
            ),
        )

    def run(self):
        LOG.debug("Exodus push begins")

        publish = self.new_publish()
        publish_id = str(publish.get("id"))
        LOG.info("Publish ID: %s", publish_id)

        for item in self.args.sync:
            LOG.debug("Processing %s", item)
            items = item.split(",")
            src, dest = items[0].strip(), items[1].strip()
            dest = dest if ":" in dest else "exodus:%s" % dest

            cmd = ["exodus-rsync", "--exodus-publish", publish_id, src, dest]
            if self.args.verbose:
                cmd.append("-" + "v" * self.args.verbose)
            if self.extra_args:
                cmd.extend(self.extra_args)

            LOG.info(" ".join(cmd))

            proc = subprocess.Popen(
                cmd,
                stdout=subprocess.PIPE,
                stderr=subprocess.STDOUT,
                universal_newlines=True,
            )
            for line in proc.stdout:
                LOG.info(line.strip())

            ret = proc.wait()
            if ret != 0:
                raise RuntimeError("Exodus push failed")

        commit_task = self.commit_publish(publish)
        self.poll_commit_completion(commit_task)

        LOG.info("Exodus push is complete")


def entry_point(args=None):
    task = ExodusPushTask(args)
    task.main()



import json
import logging
import os
import sys
from threading import Lock

import attr
from pubtools.pluggy import hookimpl, pm  # pylint: disable=wrong-import-order
from six.moves.urllib.parse import urljoin

from ..gateway import ExodusGatewaySession

LOG = logging.getLogger("pubtools-exodus")

# pylint: disable=unused-argument


class ExodusPulpHandler(ExodusGatewaySession):
    def __init__(self):
        super(ExodusPulpHandler, self).__init__()

        self.lock = Lock()

    def exodus_enabled(self):
        enable_vals = ["true", "t", "1", "yes", "y"]
        return os.getenv("EXODUS_ENABLED", "False").lower() in enable_vals

    @hookimpl
    def pulp_repository_pre_publish(self, repository, options):
        """Invoked as the first step in publishing a Pulp repository.

        This implementation adds to each config the --exodus-publish argument,
        attaching the repository to an exodus-gw publish.

        Args:
            repository (:class:`~pubtools.pulplib.Repository`):
                The repository to be published.
            options (:class:`~pubtools.pulplib.PublishOptions`):
                The options to use in publishing.
        Returns:
            options (:class:`~pubtools.pulplib.PublishOptions`):
                The adjusted options used for this publish.
        """

        if not self.exodus_enabled():
            return None

        with self.lock:
            if not self.publish:
                self.publish = self.new_publish()
                LOG.debug("created exodus-gw publish %s", self.publish["id"])

        args = (
            list(options.rsync_extra_args) if options.rsync_extra_args else []
        )
        args.append("--exodus-publish=%s" % self.publish["id"])
        return attr.evolve(options, rsync_extra_args=args)

    @hookimpl
    def task_pulp_flush(self):
        """Invoked during task execution after successful completion of all
        Pulp publishes.

        This implementation commits the active exodus-gw publish, making
        the content visible on the target CDN environment.
        """

        if not self.publish:
            LOG.debug("no exodus-gw publish to commit")
            return

        commit_url = urljoin(self.gw_url, self.publish["links"]["commit"])
        resp = self.do_request(method="POST", url=commit_url)
        commit = resp.json()

        task = self.poll_commit_completion(commit)
        LOG.info(
            "committed exodus-gw publish: %s", json.dumps(task, sort_keys=True)
        )

    @hookimpl
    def task_stop(self):
        pm.unregister(self)


@hookimpl
def task_start():
    pm.register(ExodusPulpHandler())


pm.register(sys.modules[__name__])
